<?php
class ControllerSettingSuperPageCache extends Controller {
	private $error = array();

	public function index() {

		if (!class_exists('PageCache')) {
			exit('PageCache class not found. Please make sure you\'ve run spc-install.php');
		}

		//load config/stats file
		// -- we use a config flatfile instead of OpenCart's config so the front-end can check cache settings
		//    without having to load the database abstraction layer
		$cache_dir=str_replace('system/', '', DIR_SYSTEM) . 'pagecache/';

		$config=PageCache::loadConfig();
		$this->language->load('setting/super_page_cache');

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			unset($this->request->post['spc_clear_cache']);
			if (PageCache::saveConfig($this->request->post)) {
				$this->session->data['success'] = $this->language->get('text_success');
				$this->response->redirect($this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL'));
			} else {
				$this->session->data['error'] = $this->language->get('text_error');
			}

		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');
		$data['text_cache_time_description'] = $this->language->get('text_cache_time_description');
		$data['text_cache_exclude_routes_description'] = $this->language->get('text_cache_exclude_routes_description');
		$data['text_cache_exclude_category_ids_description'] = $this->language->get('text_cache_exclude_category_ids_description');
		$data['text_cache_exclude_product_ids_description'] = $this->language->get('text_cache_exclude_product_ids_description');
		$data['text_spc_clear_cache_checkout'] = $this->language->get('text_spc_clear_cache_checkout');

		$data['entry_spc_status'] = $this->language->get('entry_spc_status');
		$data['entry_spc_cache_time'] = $this->language->get('entry_spc_cache_time');
		$data['entry_clear_cache'] = $this->language->get('entry_spc_clear_cache');
		$data['entry_spc_exclude_routes'] = $this->language->get('entry_spc_exclude_routes');
		$data['entry_spc_exclude_category_ids'] = $this->language->get('entry_spc_exclude_category_ids');
		$data['entry_spc_exclude_product_ids'] = $this->language->get('entry_spc_exclude_product_ids');
		$data['entry_spc_clear_cache_checkout'] = $this->language->get('entry_spc_clear_cache_checkout');
		$data['entry_spc_check_updates'] = $this->language->get('entry_spc_check_updates');

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		$data['button_save'] = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');
		$data['button_stats'] = $this->language->get('button_stats');

		$seconds=$this->language->get('seconds');
		$minute=$this->language->get('minute'); $minutes=$this->language->get('minutes');
		$hour=$this->language->get('hour'); $hours=$this->language->get('hours');
		$data['time_options']=array(
			30=>'30 '.$seconds,
			60=>'1 '.$minute,
			120=>'2 '.$minutes,
			180=>'3 '.$minutes,
			300=>'5 '.$minutes,
			600=>'10 '.$minutes,
			800=>'15 '.$minutes,
			1200=>'20 '.$minutes,
			1500=>'25 '.$minutes,
			1800=>'30 '.$minutes,
			2700=>'45 '.$minutes,
			3600=>'1 '.$hour,
			5400=>'1.5 '.$hours,
			7200=>'2 '.$hours,
			10800=>'3 '.$hours,
			14400=>'4 '.$hours,
			18000=>'5 '.$hours,
			21600=>'6 '.$hours,
			25200=>'7 '.$hours,
			28800=>'8 '.$hours,
			32400=>'9 '.$hours,
			36000=>'10 '.$hours,
			39600=>'11 '.$hours,
			43200=>'12 '.$hours,
			46800=>'13 '.$hours,
			50400=>'14 '.$hours,
			54000=>'15 '.$hours,
			57600=>'16 '.$hours,
			61200=>'17 '.$hours,
			64800=>'18 '.$hours,
			68400=>'19 '.$hours,
			72000=>'20 '.$hours,
			75600=>'21 '.$hours,
			79200=>'22 '.$hours,
			82800=>'23 '.$hours,
			86400=>'24 '.$hours,
			129600=>'36 '.$hours,
			172800=>'48 '.$hours,
			259200=>'72 '.$hours,
			630720000=>$this->language->get('lolwut')

		);

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->error['spc_cache_time'])) {
			$data['error_spc_cache_time'] = $this->error['spc_cache_time'];
		} else {
			$data['error_spc_cache_time'] = array();
		}

		if (isset($this->error['spc_exclude_routes'])) {
			$data['error_spc_exclude_routes'] = $this->error['spc_exclude_routes'];
		} else {
			$data['error_spc_exclude_routes'] = array();
		}

		if (isset($this->error['spc_exclude_category_ids'])) {
			$data['error_exclude_category_ids'] = $this->error['spc_exclude_category_ids'];
		} else {
			$data['error_exclude_category_ids'] = array();
		}

		if (isset($this->error['spc_exclude_product_ids'])) {
			$data['error_exclude_product_ids'] = $this->error['spc_exclude_product_ids'];
		} else {
			$data['error_exclude_product_ids'] = array();
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('text_home'),
			 'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => false
		);

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('heading_title'),
			 'href'      => $this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => ' :: '
		);

		$data['action'] = $this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL');
		$data['cancel'] = $this->url->link('extension/module', 'token=' . $this->session->data['token'], 'SSL');
		$data['stats'] = $this->url->link('setting/super_page_cache/stats', 'token=' . $this->session->data['token'], 'SSL');
		$data['text_edit'] = $this->language->get('text_edit');

		if (isset($this->request->post['spc_status'])) {
			$data['spc_status'] = $this->request->post['spc_status'];
		} else {
			$data['spc_status'] = $config['spc_status'];
		}
		if (isset($this->request->post['spc_cache_time'])) {
			$data['spc_cache_time'] = $this->request->post['spc_cache_time'];
		} else {
			$data['spc_cache_time'] = $config['spc_cache_time'];
		}

		if (isset($this->request->post['spc_exclude_routes'])) {
			$data['spc_exclude_routes'] = $this->request->post['spc_exclude_routes'];
		} else {
			$data['spc_exclude_routes'] = $config['spc_exclude_routes'];
		}
		if (isset($this->request->post['spc_exclude_category_ids'])) {
			$data['spc_exclude_category_ids'] = $this->request->post['spc_exclude_category_ids'];
		} else {
			$data['spc_exclude_category_ids'] = $config['spc_exclude_category_ids'];
		}
		if (isset($this->request->post['spc_exclude_product_ids'])) {
			$data['spc_exclude_product_ids'] = $this->request->post['spc_exclude_product_ids'];
		} else {
			$data['spc_exclude_product_ids'] = $config['spc_exclude_product_ids'];
		}
		if (isset($this->request->post['spc_check_updates'])) {
			$data['spc_check_updates'] = $this->request->post['spc_check_updates'];
		} else {
			$data['spc_check_updates'] = $config['spc_check_updates'];
		}
		if (isset($this->request->post['spc_clear_cache_checkout'])) {
			$data['spc_clear_cache_checkout'] = $this->request->post['spc_clear_cache_checkout'];
		} else {
			$data['spc_clear_cache_checkout'] = $config['spc_clear_cache_checkout'];
		}

		//check cache health
		if (!is_dir($cache_dir) || !is_writable($cache_dir)) {
			$data['error_warning']=str_replace('%DIR%', $cache_dir, $this->language->get('error_writable'));
		}

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('setting/super_page_cache.tpl', $data));
	}

	public function stats() {

		$this->language->load('setting/super_page_cache');

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('text_home'),
			 'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => false
		);

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('heading_title'),
			 'href'      => $this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => ' :: '
		);

		$data['heading_title'] = $this->language->get('stats_heading_title');
		$data['heading_cache_hits'] = $this->language->get('stats_heading_cache_hits');
		$data['heading_daily_stats'] = $this->language->get('stats_heading_daily_stats');

		$data['text_edit'] = $this->language->get('text_edit');

		$data['error_warning']='';

		//*** Load stats/config
		list($daily, $totals)=PageCache::loadStats();

		if (empty($daily)) {
			$data['error_warning']=$this->language->get('error_stats');
		}

		$data['cache_hit_pct']=0;
		if ($totals['hits']+$totals['misses'] > 0) {
			$data['cache_hit_pct']=number_format(($totals['hits'] / ($totals['hits']+$totals['misses'])) * 100, 1);
		}

		$data['totals']=$totals;
		$data['daily']=$daily;

		$data['cache_saved_time']=$this->haveSecs($totals['hits'] * .5);


		$data['action'] = $this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL');
		$data['cancel'] = $this->url->link('extension/module', 'token=' . $this->session->data['token'], 'SSL');

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('setting/super_page_cache_stats.tpl', $data));


	}

	public function clear() {
		PageCache::clearCache();

		$this->language->load('setting/super_page_cache');

		$this->document->setTitle($this->language->get('heading_title'));


		$data['text_spc_cleared'] = $this->language->get('text_spc_cleared');

		$data['error_warning']='';
		$data['heading_title']=$this->language->get('heading_title');

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('text_home'),
			 'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => false
		);

		$data['breadcrumbs'][] = array(
			 'text'      => $this->language->get('heading_title'),
			 'href'      => $this->url->link('setting/super_page_cache', 'token=' . $this->session->data['token'], 'SSL'),
			 'separator' => ' :: '
		);

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('setting/super_page_cache_cleared.tpl', $data));


	}

	protected function validate() {
		$this->request->post['spc_cache_time']=trim($this->request->post['spc_cache_time']);

		if (!preg_match('/^[0-9]+$/', $this->request->post['spc_cache_time'] )) {
			$this->error['spc_cache_time'] = $this->language->get('error_spc_cache_time');
		}

		if ($this->request->post['spc_exclude_product_ids']) {
			$outstr='';
			foreach (explode(',', $this->request->post['spc_exclude_product_ids']) as $id) {
				$id=(int)trim($id);
				if ($id) {
					$outstr.=$id.',';
				}
			}
			$this->request->post['spc_exclude_product_ids']=trim($outstr,',');
		}
		if ($this->request->post['spc_exclude_category_ids']) {
			$outstr='';
			foreach (explode(',', $this->request->post['spc_exclude_category_ids']) as $id) {
				$id=(int)trim($id);
				if ($id) {
					$outstr.=$id.',';
				}
			}
			$this->request->post['spc_exclude_category_ids']=trim($outstr,',');
		}
		if ($this->request->post['spc_exclude_routes']) {
			$outstr='';
			foreach (explode(',', $this->request->post['spc_exclude_routes']) as $id) {
				$id=trim($id);
				if ($id) {
					$outstr.=$id.',';
				}
			}
			$this->request->post['spc_exclude_routes']=trim($outstr,',');
		}

		if (isset($this->request->post['clear_cache']) && $this->request->post['clear_cache']) {
			PageCache::clearCache();
		}

		if (!$this->error) {
			return true;
		} else {
			return false;
		}
	}


	protected function haveSecs($secs) {
		$units = array(
			'century'=>36500*24*3600,
			'decade'=>3650*24*3600,
			'year'=>365*24*3600,
			'month'=>30*24*3600,
			'week'=>7*24*3600,
			'day'=> 24*3600,
			'hour'=>3600,
			'minute'=>60,
			'second'=>1,
		);

		if (!$secs) {
			return "No time at all";
		}

		$str='';
		foreach ($units as $name=>$divisor) {
			if ($quot = intval($secs / $divisor) ) {
				$str.=$quot.' '.$name;
				$str.=(abs($quot) > 1 ? "s" : "") . ", ";
				$secs-=$quot * $divisor;
			}
		}

		return substr($str, 0, -2);
	}
}