<?php

/**
 * Class PageCache
 *
 * version 2.0.2
 *
 * Copyright 2014: This is commercial software and may not be distributed
 *
 * @author Ben Wampler ben@forlent.com
 *
 */
class PageCache {
	private static $__route='unknown';
	private static $__route_token=false;
	private static $__cache_path;
	private static $__cache_time;
	private static $__request;
	private static $__keyed_route=false;
	private static $__db;
	private static $__config=array();
	private static $__exclude_routes=array();
	private static $__exclude_product_ids=array();
	private static $__exclude_category_ids=array();

	public static $force_disable=false;

	private static $__product_id=false;
	private static $__category_path=false;
	private static $__manufacturer_id=false;
	private static $__information_id=false;


	/**
	 * Initializes class with OpenCart data
	 *
	 */
	public static function setSession() {
		if (isset($_COOKIE['PHPSESSID']) && !preg_match('/^[a-z0-9]+$/', $_COOKIE['PHPSESSID'])) {
			unset($_COOKIE['PHPSESSID']);
		}

		if (!session_id()) {
			session_start();
		}

		self::$__request=(!empty($_GET) ? $_GET : array());
		if (!empty($_POST) && is_array($_POST)) self::$__request+=$_POST;
		self::$__cache_path=str_replace('/system/', '/pagecache/', DIR_SYSTEM);

		//load config, disable module if any problems
		$disabled=self::$force_disable;

		self::__loadConfig();
		if (!self::$__config['spc_status']) {
			$disabled=true;
			self::__log('SPC manually disabled');
		} else {
			if (!is_dir(self::$__cache_path) || !is_writable(self::$__cache_path)) {
				$disabled=true;
			}
		}

		if (!$disabled) {
			self::$__exclude_routes[]='product/product/upload';
			self::$__exclude_routes[]='product/product/captcha';
			self::$__exclude_routes[]='product/compare';
			self::$__exclude_routes[]='product/compare/add';
			self::$__exclude_routes[]='information/contact';
			self::$__exclude_routes[]='information/contact/captcha';

			$excluded_routes=self::$__config['spc_exclude_routes'];
			foreach (explode(',', $excluded_routes) as $route) {
				if (!$route) continue;
				self::$__exclude_routes[]=$route;
			}

			$excluded_products=self::$__config['spc_exclude_product_ids'];
			foreach (explode(',', $excluded_products) as $product_id) {
				if (!$product_id) continue;
				self::$__exclude_product_ids[]=$product_id;
			}

			$excluded_categories=self::$__config['spc_exclude_category_ids'];
			foreach (explode(',', $excluded_categories) as $category_id) {
				if (!$category_id) continue;
				self::$__exclude_category_ids[]=$category_id;
			}

			self::$__cache_time=self::$__config['spc_cache_time'];
		}


		self::$force_disable=$disabled;
	}

	/**
	 * @return bool|string
	 */
	public static function mayCache() {

		if (self::$force_disable==true) {
			//cache disabled
			return false;
		}
		if (!empty($_SESSION['cart']) || !empty($_SESSION['customer_id']) || !empty(self::$__request->post)) {
			self::__log('Not cached: items in cart, POST, or user logged in');
			return false;
		}

		if (!empty(self::$__request['tracking'])) {
			self::__log('Not cached: affiliate link');
			return false;
		}

		if (isset(self::$__request['redirect']) && self::$__request['redirect']) {
			self::__log('Not cached: redirect');
			$disabled=true;
		}

		//assign routes
		if (isset(self::$__request['route']) && self::$__request['route']) {
			self::$__route=self::$__request['route'];
		} elseif (isset(self::$__request['_route_']) && self::$__request['_route_']) {
			self::$__route=self::__decodeSEOUrl(self::$__request['_route_']);
		}

		if (!isset(self::$__request['route']) && !isset(self::$__request['_route_'])) {
			//no routes at all, must be index
			self::$__route='common/home';
		}

		if (!preg_match('/^(common|index|feed|information|product)/', self::$__route)) {
			//only cache common, feed, information, or product
			self::__log('Not a cacheable page ('.self::$__route.')');
			return false;
		}

		//check for excluded routes
		foreach (self::$__exclude_routes as $route) {
			if (strtolower(self::$__route)==strtolower($route)) {
				self::__log('Not cached: manually excluded route ('.self::$__route.')');
				return false;
			}
		}

		//check for excluded products
		if (!empty(self::$__exclude_product_ids)) {
			if (isset(self::$__request['product_id']) && self::$__request['product_id']) {
				$current_product_id=(int)self::$__request['product_id'];
			} else {
				$current_product_id=(int)self::$__product_id;
			}

			foreach (self::$__exclude_product_ids as $product_id) {
				if ($current_product_id==(int)$product_id) {
					self::__log('Not cached: manually excluded product ('.$product_id.')');
					return false;
				}
			}
		}

		//check for excluded categories
		if (!empty(self::$__exclude_category_ids)) {

			if (isset(self::$__request['path']) && self::$__request['path']) {
				$path=self::$__request['path'];
			} else {
				$path=self::$__category_path;
			}

			$path_items=explode('_', $path);
			foreach ($path_items as $path_id) {
				foreach (self::$__exclude_category_ids as $category_id) {
					if ((int)$path_id==(int)$category_id) {
						self::__log('Not cached: manually excluded category ('.$category_id.')');
						return false;
					}
				}
			}
		}
		return true;

	}

	/**
	 * Loads cached version of current page if available
	 */
	public static function findCachedVersion() {
		$route_token=self::routeToken();

		$filename=self::$__cache_path . '/' . md5($route_token) . '.spc';
		if (file_exists($filename)) {
			self::__checkTimezone();

			$data=unserialize(file_get_contents($filename));
			if ((time() - $data['created']) > self::$__cache_time) {
				@unlink($filename);
			} else {
				if ($data['spc_proper_url']!==false) {
					//compatibility with my OpenCart SEO Link Deduplicator plugin (only $8 at CodeCanyon.net!)
					$actual_url=$_SERVER['REQUEST_URI'];
					$proper_url='/'.$data['spc_proper_url'];

					$requests=self::$__request;
					unset($requests['_route_']); unset($requests['route']); unset($requests['path']);
					if (!empty($requests)) {
						$do_redir=false;
						foreach ($requests as $key=>$val) {
							if ($key=='page' && $val==1) {
								unset($requests[$key]); $do_redir=true;
							} elseif ($key=='sort' && $val=='p.sort_order') {
								unset($requests[$key]); $do_redir=true;
								if (isset($requests['order']) && $requests['order']=='ASC') {
									unset($requests['order']);
								}
							} elseif ($key=='limit' && $val==$data['config_catalog_limit']) {
								unset($requests[$key]); $do_redir=true;
							}
						}
						if ($do_redir) {
							$can_url=$proper_url . (!empty($requests) ? '?'.http_build_query($requests) : '');
							header("HTTP/1.1 301 Moved Permanently");
							header("Location: ".$can_url);
							exit;
						} else {
							$proper_url.=(!empty($requests) ? '?'.http_build_query($requests) : '');
						}
					}

					if ($actual_url!=$proper_url && !preg_match('@product_id=[0-9]+@', $proper_url) && !preg_match('@path=[0-9]+@', $proper_url) && !preg_match('/information_id=[0-9]+/', $proper_url) && preg_match('/^[A-Za-z0-9_\.,-]+$/', $proper_url)) {
						self::__log("Dedupe Forwarding: $actual_url to {$data['spc_proper_url']}");
						header("HTTP/1.1 301 Moved Permanently");
						header("Location: ".$proper_url);
						exit;
					}
				}

				if (!headers_sent()) {
					foreach ($data['headers'] as $header) {
						header($header, true);
					}
				}

				echo self::__gzipEncode($data['output'], $data['config_compression']);
				flush();

				self::__stats('hit');
				self::__log('HIT: '.$route_token);
				exit;
			}
		}

		self::__log('MISS: '.$route_token);
		self::__stats('miss');
	}

	/**
	 * Stores cached version into local page cache
	 *
	 * @param $headers raw output headers
	 * @param $output raw output html
	 */
	public static function saveCachedVersion(&$headers, &$output) {
		if (!self::mayCache()) return;

		$config=$GLOBALS['registry']->get('config');
		$route_token=self::routeToken();
		$filename=self::$__cache_path . '/' . md5($route_token) . '.spc';
		file_put_contents(	$filename,
			 			serialize(array(	'created'=>time(),
							 			'headers'=>$headers,
							 			'output'=>$output,
										'config_catalog_limit'=>$config->get('config_catalog_limit'),
										'config_compression'=>$config->get('config_compression'),
							 			'spc_proper_url'=>(isset($GLOBALS['spc_proper_url']) ? $GLOBALS['spc_proper_url'] : false))));

	}

	/**
	 * Deletes all cached items in the cache_path folder
	 */
	public static function clearCache() {
		$files=glob(self::$__cache_path.'/*.spc');

		$ctr=0;
		if (!empty($files)) {
			foreach ($files as $file) {
				if (unlink($file)) {
					$ctr++;
				}
			}
		}

		self::__log('Cache cleared. Deleted '.$ctr.' item(s).');
	}

	/**
	 * Returns gzip-encoded version IF
	 *
	 * @param $output
	 * @param $level
	 * @return mixed
	 */
	private static function __gzipEncode($output, $level) {
		if (isset($_SERVER['HTTP_ACCEPT_ENCODING'])) {
			if (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false) {
				$use_encoding = 'gzip';
			}

			if (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'x-gzip') !== false) {
				$use_encoding = 'x-gzip';
			}
		}

		if (!isset($use_encoding) || !$use_encoding || !extension_loaded('zlib') || ini_get('zlib.output_compression') || headers_sent() || connection_status()) {
			return $output;
		}

		if ($use_encoding) {
			header('Content-Encoding: ' . $use_encoding);
			return gzencode($output, $level);
		}

		return $output;
	}

	/**
	 * @return mixed|string
	 */
	private static function routeToken() {
		if (self::$__route_token) return self::$__route_token;

		$route_token=self::$__keyed_route.'-'.self::$__route;

		foreach (self::$__request as $key=>$get) {
			if ($key=='route' || $key=='_route_') continue;
			$route_token.='--'.$key.$get;
		}

		if (isset($_SESSION['currency']) && $_SESSION['currency']) {
			//currency specified; we need to cache in this currency
			$route_token.='--'.$_SESSION['currency'];
		}
		if (isset($_SESSION['language']) && $_SESSION['language']) {
			//currency specified; we need to cache in this currency
			$route_token.='--'.$_SESSION['language'];
		}

		self::$__route_token=preg_replace('/[^A-Za-z0-9]/','_', $route_token);

		return self::$__route_token;
	}

	/**
	 * Stores a message to the SPC log
	 *
	 * @param $text string text to log
	 */
	private static function __log($text) {
		self::__checkTimezone();

		$pf=fopen(DIR_SYSTEM . 'logs/spc.log', 'a');
		fwrite($pf, date('Y-m-d H:i:s').': '.$text."\n");
		fclose($pf);
	}

	/**
	 * Increments counters for stats
	 *
	 * @param $type
	 */
	private static function __stats($type) {
		self::__checkTimezone();

		$stats_file=self::$__cache_path . 's'.date('Y-m-d').'.stats';

		$arr=array();
		if (file_exists($stats_file)) {
			$arr=file_get_contents($stats_file, 'a');
		}

		if (!empty($arr)) {
			$arr=@unserialize($arr);
		}

		if (!isset($arr[date('Y-m-d')][$type])) {
			$arr[date('Y-m-d')][$type]=1;
		} else {
			$arr[date('Y-m-d')][$type]++;
		}
		@file_put_contents($stats_file, serialize($arr), LOCK_EX);
	}


	/**
	 * Loads the spc.cache config portion into self::$__config;
	 *
	 * @return void
	 */
	private static function __loadConfig() {
		$stats_file=self::configFileName();

		$arr=array();
		if (file_exists($stats_file)) {
			$arr=unserialize(file_get_contents($stats_file, 'a'));
		}

		if (!empty($arr['config'])) {
			self::$__config=@$arr['config'];
		}

		//default values
		if (!isset(self::$__config['spc_status'])) {
			self::$__config=array(
				'spc_status'=>0,
				'spc_cache_time'=>600,
				'spc_exclude_routes'=>'',
				'spc_exclude_product_ids'=>'',
				'spc_exclude_category_ids'=>'',
				'spc_clear_cache_checkout'=>false,
				'spc_check_updates'=>true,
			);
		}

	}

	/**
	 * Getter for __loadConfig()
	 *
	 * @return array
	 */
	public static function loadConfig() {
		self::__loadConfig();
		return self::$__config;
	}

	/**
	 * Stores updated config
	 *
	 * @param $config array
	 * @return bool
	 */
	public static function saveConfig($config) {
		$config_file=self::configFileName();

		$stats=array();
		if (file_exists($config_file)) {
			$data=file_get_contents($config_file);
			$stats=unserialize($data);
		} else {
			self::__log('Config file doesn\'t exist');
		}

		$stats['config']=$config;

		return file_put_contents($config_file, serialize($stats), LOCK_EX);
	}

	/**
	 * Reads a single configuration option from SPC config
	 *
	 * @param $var
	 * @return mixed
	 */
	public static function readConfig($var) {
		if (isset(self::$__config[$var])) {
			return self::$__config[$var];
		} else {
			return false;
		}
	}

	/**
	 * Returns a string with the full path to the configuration file
	 *
	 * @return string
	 */
	public static function configFileName() {
		return self::$__cache_path . 'spc.config';
	}


	/**
	 * loadStats()
	 *
	 * Loads, assembles, and tabulates all stats from pagecache stats files
	 *
	 * @return array
	 */
	public static function loadStats() {
		$stats_files=glob(self::$__cache_path.'*.stats');

		$daily=array(); $totals=array('hits'=>0, 'misses'=>0);
		foreach ($stats_files as $stats_file) {
			if (file_exists($stats_file)) {
				$arr=file_get_contents($stats_file);
				$stats=unserialize($arr);
				if (empty($stats) || !is_array($stats)) continue;

				$daily+=$stats;

				//get total cache hits
				foreach ($stats as $date=>$arr) {
					if (isset($arr['hit'])) $totals['hits']+=(int)$arr['hit'];
					if (isset($arr['miss'])) $totals['misses']+=(int)$arr['miss'];
				}
			}

		}

		return array(
			 $daily,
			 $totals
		);

	}


	/**
	 * Uses url_alias table to decode an SEO'd url into its route and ID components
	 *
	 * @param $route
	 * @return bool|string
	 */
	private static function __decodeSEOUrl($route) {
		$product_id=false; $manufacturer_id=false; $information_id=false; $path=false;

		if (empty(self::$__db)) {
			//load DB connection on-the-fly only if needed to reduce overhead
			self::$__db = new fastDB(DB_DRIVER, DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE);
		}

		$parts=explode('/', $route);
		foreach ($parts as $part) {
			$res = self::$__db->query("SELECT * FROM " . DB_PREFIX . "url_alias WHERE keyword = '" . self::$__db->escape($part) . "'");
			if (!empty($res->rows)) {
				$url = explode('=', $res->row['query']);

				if ($url[0] == 'product_id') {
					$product_id = $url[1];
					self::$__product_id=$url[1];
				}

				if ($url[0] == 'category_id') {
					if (!isset(self::$__request['path'])) {
						$path = $url[1];
					} else {
						$path .= '_' . $url[1];
					}
					self::$__category_path=$path;
				}

				if ($url[0] == 'manufacturer_id') {
					$manufacturer_id=$url[1];
					self::$__manufacturer_id=$url[1];
				}

				if ($url[0] == 'information_id') {
					$information_id=$url[1];
					self::$__information_id=$url[1];
				}

			} else {
				return false;
			}
		}


		if ($product_id) {
			$route='product/product';
			self::$__keyed_route=$route.$product_id;
		} elseif ($path) {
			$route='product/category';
			self::$__keyed_route=$route.$path;
		} elseif ($manufacturer_id) {
			$route='product/manufacturer/info';
			self::$__keyed_route=$route.$manufacturer_id;
		} elseif ($information_id) {
			$route='information/information';
			self::$__keyed_route=$route.$information_id;
		}

		return $route;
	}

	/**
	 * If no timezone is set in php.ini, set to UTC to avoid PHP notices
	 *
	 * @returns void
	 */
	private static function __checkTimezone() {
		if (!ini_get('date.timezone')) {
			date_default_timezone_set('UTC');
		}
	}
}


class fastDB {
	private $db;

	public function __construct($driver, $hostname, $username, $password, $database) {
		$file = DIR_SYSTEM . '/' . 'library/db/' . $driver . '.php';

		if (file_exists($file)) {
			require_once($file);
		}

		$class = 'DB\\' . $driver;
		if (class_exists($class)) {
			$this->db = new $class($hostname, $username, $password, $database);
		} else {
			exit('Error: Could not load database driver ' . $driver . '!');
		}
	}

	public function query($sql) {
		return $this->db->query($sql);
	}

	public function escape($value) {
		return $this->db->escape($value);
	}

	public function countAffected() {
		return $this->db->countAffected();
	}

	public function getLastId() {
		return $this->db->getLastId();
	}
}
